import os
from .providers.smtp_provider import SMTPProvider
from .providers.sendgrid_provider import SendGridProvider
from .providers.mailgun_provider import MailgunProvider
from .providers.ses_provider import SESProvider
from .providers.postmark_provider import PostmarkProvider
from .providers.brevo_provider import BrevoProvider
from dotenv import load_dotenv

load_dotenv()

class MailerFactory:
    @staticmethod
    def get_provider():
        provider_name = os.getenv("MAIL_MAILER", "smtp").lower()

        if provider_name == "smtp":
            return SMTPProvider(
                host=os.getenv("MAIL_HOST"),
                port=int(os.getenv("MAIL_PORT", 587)),
                username=os.getenv("MAIL_USERNAME"),
                password=os.getenv("MAIL_PASSWORD"),
                use_tls=os.getenv("MAIL_TLS_ENCRYPTION", "True") == "True",
                use_ssl=os.getenv("MAIL_SSL_ENCRYPTION", "False") == "True",
            )
        elif provider_name == "sendgrid":
            return SendGridProvider(
                api_key=os.getenv("MAIL_API_KEY"),
                endpoint=os.getenv("MAIL_ENDPOINT"),
            )
        elif provider_name == "mailgun":
            return MailgunProvider(
                api_key=os.getenv("MAIL_API_KEY"),
                endpoint=os.getenv("MAIL_ENDPOINT"),
            )
        elif provider_name == "ses":
            return SESProvider(
                aws_access_key_id=os.getenv("MAIL_AWS_ACCESS_KEY_ID"),
                aws_secret_access_key=os.getenv("MAIL_AWS_SECRET_ACCESS_KEY"),
                region_name=os.getenv("MAIL_AWS_REGION"),
            )
        elif provider_name == "postmark":
            return PostmarkProvider(
                server_token=os.getenv("MAIL_API_KEY"),
                endpoint=os.getenv("MAIL_ENDPOINT"),
            )
        elif provider_name == "brevo":
            return BrevoProvider(
                api_key=os.getenv("MAIL_API_KEY"),
                endpoint=os.getenv("MAIL_ENDPOINT"),
            )
        else:
            raise ValueError(f"Unsupported mail provider: {provider_name}")


import os
import pytest
from mailbridge.mailer_factory import MailerFactory
from mailbridge.providers.smtp_provider import SMTPProvider
from mailbridge.providers.sendgrid_provider import SendGridProvider
from mailbridge.providers.mailgun_provider import MailgunProvider
from mailbridge.providers.ses_provider import SESProvider
from mailbridge.providers.postmark_provider import PostmarkProvider
from mailbridge.providers.brevo_provider import BrevoProvider


@pytest.mark.parametrize(
    "provider_name,expected_class,env_vars",
    [
        (
            "smtp",
            SMTPProvider,
            {
                "MAIL_MAILER": "smtp",
                "MAIL_HOST": "smtp.test.com",
                "MAIL_PORT": "587",
                "MAIL_USERNAME": "user@test.com",
                "MAIL_PASSWORD": "secret",
                "MAIL_ENCRYPTION": "True",
            },
        ),
        (
            "sendgrid",
            SendGridProvider,
            {
                "MAIL_MAILER": "sendgrid",
                "MAIL_API_KEY": "SG.KEY",
                "MAIL_ENDPOINT": "https://api.sendgrid.com/v3/mail/send",
            },
        ),
        (
            "mailgun",
            MailgunProvider,
            {
                "MAIL_MAILER": "mailgun",
                "MAIL_API_KEY": "MG.KEY",
                "MAIL_ENDPOINT": "https://api.mailgun.net/v3/test/messages",
            },
        ),
        (
            "ses",
            SESProvider,
            {
                "MAIL_MAILER": "ses",
                "MAIL_AWS_ACCESS_KEY_ID": "AKIAXXX",
                "MAIL_AWS_SECRET_ACCESS_KEY": "SECRET",
                "MAIL_AWS_REGION": "us-east-1",
            },
        ),
        (
            "postmark",
            PostmarkProvider,
            {
                "MAIL_MAILER": "postmark",
                "MAIL_API_KEY": "PM.KEY",
                "MAIL_ENDPOINT": "https://api.postmarkapp.com/email",
            },
        ),
        (
            "brevo",
            BrevoProvider,
            {
                "MAIL_MAILER": "brevo",
                "MAIL_API_KEY": "BREVO.KEY",
                "MAIL_ENDPOINT": "https://api.brevo.com/v3/smtp/email",
            },
        ),
    ],
)
def test_mailer_factory_get_provider(provider_name, expected_class, env_vars, mocker):
    # Save original environment
    original_env = os.environ.copy()

    # Override env vars for test
    os.environ.update(env_vars)

    provider = MailerFactory.get_provider()

    assert isinstance(provider, expected_class), f"{provider_name} did not return expected provider"

    # Restore environment
    os.environ.clear()
    os.environ.update(original_env)


def test_mailer_factory_invalid_provider(monkeypatch):
    monkeypatch.setenv("MAIL_MAILER", "unknown")

    with pytest.raises(ValueError) as exc_info:
        MailerFactory.get_provider()

    assert "Unsupported mail provider" in str(exc_info.value)



from .mailer_factory import MailerFactory

class Mail:
    @staticmethod
    def send(to: str, subject: str, body: str, from_email: str = None) -> bool:
        """
        Sends an email using the configured provider from environment variables.

        :param to: Recipient email
        :param subject: Email subject
        :param body: HTML body of the email
        :param from_email: Optional sender email; if None, provider default is used
        :return: True if email was sent successfully, False otherwise
        """
        try:
            provider = MailerFactory.get_provider()
            return provider.send(to=to, subject=subject, body=body, from_email=from_email)
        except Exception as e:
            # Optional: log error (e.g., with logging library)
            print(f"[MailBridge] Error sending email: {e}")
            return False



import boto3
from .provider_interface import ProviderInterface

class SESProvider(ProviderInterface):
    def __init__(self, aws_access_key_id, aws_secret_access_key, region_name):
        self.client = boto3.client(
            "ses",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name,
        )

    def send(self, to, subject, body, from_email=None):
        resp = self.client.send_email(
            Source=from_email,
            Destination={"ToAddresses": [to]},
            Message={
                "Subject": {"Data": subject},
                "Body": {"Html": {"Data": body}},
            },
        )
        return resp["ResponseMetadata"]["HTTPStatusCode"] == 200


import requests
from .provider_interface import ProviderInterface

class BrevoProvider(ProviderInterface):
    def __init__(self, api_key, endpoint):
        self.api_key = api_key
        self.endpoint = endpoint

    def send(self, to, subject, body, from_email=None):
        data = {
            "sender": {"email": from_email},
            "to": [{"email": to}],
            "subject": subject,
            "htmlContent": body,
        }

        headers = {
            "api-key": self.api_key,
            "Content-Type": "application/json",
            "Accept": "application/json"
        }

        resp = requests.post(self.endpoint, json=data, headers=headers)
        resp.raise_for_status()
        return resp.status_code in (200, 201, 202)


import smtplib
from email.mime.text import MIMEText
from .provider_interface import ProviderInterface

class SMTPProvider(ProviderInterface):
    def __init__(self, host: str, port: int, username: str, password: str, use_tls: bool = True, use_ssl: bool = False):
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.use_tls = use_tls
        self.use_ssl = use_ssl

    def send(self, to: str, subject: str, body: str, from_email: str = None) -> bool:
        msg = MIMEText(body, "html")
        msg["Subject"] = subject
        msg["From"] = from_email or self.username
        msg["To"] = to

        try:
            if self.use_ssl:
                # SMTP over SSL (port 465)
                with smtplib.SMTP_SSL(self.host, self.port) as server:
                    server.login(self.username, self.password)
                    server.send_message(msg)
            else:
                # SMTP with STARTTLS (port 587)
                with smtplib.SMTP(self.host, self.port) as server:
                    server.ehlo()  # Initialize connection
                    if self.use_tls:
                        server.starttls()
                        server.ehlo()
                    if self.username and self.password:
                        server.login(self.username, self.password)
                    server.send_message(msg)
            return True
        except smtplib.SMTPException as e:
            print(f"[MailBridge] Error sending email: {e}")
            return False



import pytest
from mailbridge.providers.ses_provider import SESProvider

@pytest.fixture
def ses_provider(mocker):
    mock_client = mocker.Mock()
    mock_client.send_email.return_value = {"ResponseMetadata": {"HTTPStatusCode": 200}}

    mock_boto3 = mocker.patch("boto3.client", return_value=mock_client)
    provider = SESProvider("AKIAXXX", "SECRET", "us-east-1")

    return provider

def test_send_email_ses(ses_provider):
    result = ses_provider.send("to@test.com", "Subject", "<p>Body</p>", "from@test.com")

    assert result is True



from abc import ABC, abstractmethod

class ProviderInterface(ABC):
    @abstractmethod
    def send(self, to:str, subject: str, body: str, from_email: str = None):
        """Send an email through this provider."""
        pass


import pytest
from mailbridge.providers.postmark_provider import PostmarkProvider

@pytest.fixture
def postmark_provider():
    return PostmarkProvider(server_token="PM.TEST", endpoint="https://api.postmarkapp.com/email")

def test_send_email_postmark(mocker, postmark_provider):
    mock_post = mocker.patch("requests.post")
    mock_response = mocker.Mock(status_code=200)
    mock_post.return_value = mock_response

    result = postmark_provider.send("to@test.com", "Hi", "<p>Postmark body</p>", "from@test.com")

    assert result is True
    mock_post.assert_called_once_with(
        "https://api.postmarkapp.com/email",
        json=mocker.ANY,
        headers=mocker.ANY
    )



import requests
from .provider_interface import ProviderInterface

class PostmarkProvider(ProviderInterface):
    def __init__(self, server_token, endpoint):
        self.server_token = server_token
        self.endpoint = endpoint

    def send(self, to, subject, body, from_email=None):
        data = {
            "From": from_email,
            "To": to,
            "Subject": subject,
            "HtmlBody": body,
        }

        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "X-Postmark-Server-Token": self.server_token,
        }

        resp = requests.post(self.endpoint, json=data, headers=headers)
        resp.raise_for_status()
        return resp.status_code == 200


import pytest
from mailbridge.providers.brevo_provider import BrevoProvider

@pytest.fixture
def brevo_provider():
    return BrevoProvider(api_key="BREVO.KEY", endpoint="https://api.brevo.com/v3/smtp/email")

def test_send_email_brevo(mocker, brevo_provider):
    mock_post = mocker.patch("requests.post")
    mock_response = mocker.Mock(status_code=201)
    mock_post.return_value = mock_response

    result = brevo_provider.send("to@test.com", "Hi", "<p>Brevo body</p>", "from@test.com")

    assert result is True
    mock_post.assert_called_once_with(
        "https://api.brevo.com/v3/smtp/email",
        json=mocker.ANY,
        headers=mocker.ANY
    )



import pytest
from unittest.mock import MagicMock, patch
from mailbridge.mail import Mail


@patch("mailbridge.mailer_factory.MailerFactory.get_provider")
def test_mail_send_success(mock_get_provider):
    """Test that Mail.send() calls the provider's send() method correctly."""
    mock_provider = MagicMock()
    mock_provider.send.return_value = True
    mock_get_provider.return_value = mock_provider

    result = Mail.send(
        to="user@example.com",
        subject="Welcome",
        body="<h1>Hello!</h1>",
        from_email="noreply@example.com",
    )

    # Verify result and interactions
    assert result is True
    mock_provider.send.assert_called_once_with(
        to="user@example.com",
        subject="Welcome",
        body="<h1>Hello!</h1>",
        from_email="noreply@example.com",
    )

@patch("mailbridge.mailer_factory.MailerFactory.get_provider")
def test_mail_send_failure(mock_get_provider):
    """Test that Mail.send() returns False if provider.send() raises an error."""
    mock_provider = MagicMock()
    mock_provider.send.side_effect = Exception("Something went wrong")
    mock_get_provider.return_value = mock_provider

    result = Mail.send(
        to="user@example.com",
        subject="Fail Test",
        body="Error expected",
        from_email="noreply@example.com",
    )

    assert result is False
    mock_provider.send.assert_called_once()

@patch("mailbridge.mailer_factory.MailerFactory.get_provider")
def test_mail_send_without_from_email(mock_get_provider):
    """Ensure Mail.send works even if from_email is omitted."""
    mock_provider = MagicMock()
    mock_provider.send.return_value = True
    mock_get_provider.return_value = mock_provider

    result = Mail.send(
        to="user@example.com",
        subject="No sender",
        body="Testing no from_email",
    )

    assert result is True
    mock_provider.send.assert_called_once_with(
        to="user@example.com",
        subject="No sender",
        body="Testing no from_email",
        from_email=None,
    )


import requests
from .provider_interface import ProviderInterface

class SendGridProvider(ProviderInterface):
    def __init__(self, api_key, endpoint):
        self.api_key = api_key
        self.endpoint = endpoint

    def send(self, to:str, subject: str, body: str, from_email: str = None):
        data = {
            "personalizations": [{"to": [{"email": to}]}],
            "from": {"email": from_email},
            "subject": subject,
            "content": [{"type": "text/html", "value": body}],
        }

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        resp = requests.post(self.endpoint, json=data, headers=headers)
        resp.raise_for_status()
        return resp.status_code == 202


import pytest
from mailbridge.providers.mailgun_provider import MailgunProvider

@pytest.fixture
def mailgun_provider():
    return MailgunProvider(
        api_key="MG.TESTKEY",
        endpoint="https://api.mailgun.net/v3/test/messages"
    )

def test_send_email_mailgun(mocker, mailgun_provider):
    mock_post = mocker.patch("requests.post")
    mock_response = mocker.Mock(status_code=200)
    mock_post.return_value = mock_response

    result = mailgun_provider.send("to@test.com", "Hi", "<p>Mailgun body</p>", "from@test.com")

    assert result is True
    mock_post.assert_called_once_with(
        "https://api.mailgun.net/v3/test/messages",
        auth=("api", "MG.TESTKEY"),
        data=mocker.ANY
    )



import requests
from .provider_interface import ProviderInterface

class MailgunProvider(ProviderInterface):
    def __init__(self, api_key, endpoint):
        self.api_key = api_key
        self.endpoint = endpoint

    def send(self, to, subject, body, from_email=None):
        data = {
            "from": from_email,
            "to": [to],
            "subject": subject,
            "html": body,
        }

        resp = requests.post(
            self.endpoint,
            auth=("api", self.api_key),
            data=data,
        )
        resp.raise_for_status()
        return resp.status_code == 200


import pytest
from mailbridge.providers.smtp_provider import SMTPProvider

@pytest.fixture
def smtp_provider():
    return SMTPProvider(
        host="smtp.test.com",
        port=587,
        username="user@test.com",
        password="secret",
        use_tls=True,
    )

def test_send_email_smtp(mocker, smtp_provider):
    mock_smtp = mocker.patch("smtplib.SMTP", autospec=True)
    instance = mock_smtp.return_value.__enter__.return_value

    result = smtp_provider.send(
        to="receiver@test.com",
        subject="Test Subject",
        body="<b>Hello SMTP</b>",
        from_email="sender@test.com"
    )

    assert result is True
    instance.send_message.assert_called_once()
    instance.login.assert_called_once_with("user@test.com", "secret")


import pytest
from mailbridge.providers.sendgrid_provider import SendGridProvider

@pytest.fixture
def sendgrid_provider():
    return SendGridProvider(
        api_key="SG.TESTKEY",
        endpoint="https://api.sendgrid.com/v3/mail/send"
    )

def test_send_email_sendgrid(mocker, sendgrid_provider):
    mock_post = mocker.patch("requests.post")
    mock_response = mocker.Mock(status_code=202)
    mock_post.return_value = mock_response

    result = sendgrid_provider.send("to@test.com", "Hello", "<p>Body</p>", "from@test.com")

    assert result is True
    mock_post.assert_called_once_with(
        "https://api.sendgrid.com/v3/mail/send",
        json=mocker.ANY,
        headers=mocker.ANY
    )